@@ -21,23 +21,46 @@
 */
 
 #include "kglobalaccel.h"
-#include <X11/X.h>
-#include <X11/Xlib.h>
-#include <X11/keysym.h>
-#include <string.h>
-
 #include <qkeycode.h>
 #include <qlayout.h>
 #include <qpainter.h>
 #include <qapplication.h>
 #include <qdrawutil.h>
-#include <qmessagebox.h>
 
 #include <kapp.h>
+#include <klocale.h>
+#include <kconfig.h>
+
+#include <X11/X.h>
+#include <X11/Xlib.h>
+#include <X11/keysym.h>
+#include <string.h>
+
+template class QDict<KKeyEntry>;
 
 // NOTE ABOUT CONFIGURATION CHANGES
 // Test if keys enabled because these keys have made X server grabs
 
+class  KGlobalAccelPrivate : public QWidget
+{
+public:
+    KGlobalAccelPrivate( KGlobalAccel* a)
+	:QWidget(), accel( a )
+    {
+	kapp->installX11EventFilter( this );
+    }
+
+protected:
+    bool x11Event( XEvent * e )
+    {
+	return accel->x11EventFilter( e );
+    }
+
+
+private:
+    KGlobalAccel* accel;
+};
+
 KGlobalAccel::KGlobalAccel(bool _do_not_grab)
  : QObject(), aKeyDict(100)
 {
@@ -45,20 +68,26 @@ KGlobalAccel::KGlobalAccel(bool _do_not_grab)
 	bEnabled = true;
 	aGroup = "Global Keys";
 	do_not_grab =_do_not_grab;
+	d = 0;
+	if ( !do_not_grab )
+	    d = new KGlobalAccelPrivate( this );
 }
 
-KGlobalAccel::KGlobalAccel(QWidget * parent, const char * name, bool _do_not_grab)
+KGlobalAccel::KGlobalAccel(QWidget * parent, const char *name, bool _do_not_grab)
     : QObject(parent, name), aKeyDict(100) {
     	aAvailableId = 1;
 	bEnabled = true;
 	aGroup = "Global Keys";
 	do_not_grab =_do_not_grab;
-
+	d = 0;
+	if ( !do_not_grab )
+	    d = new KGlobalAccelPrivate( this );
 }
 
 KGlobalAccel::~KGlobalAccel()
 {
 	setEnabled( false );
+	delete d;
 }
 
 void KGlobalAccel::clear()
@@ -67,17 +96,15 @@ void KGlobalAccel::clear()
 	aKeyDict.clear();
 }
 
-void KGlobalAccel::connectItem( const char * action,
-				const QObject* receiver, const char* member,
+void KGlobalAccel::connectItem( const QString& action,
+				const QObject* receiver, const char *member,
 				bool activate )
 {
     KKeyEntry *pEntry = aKeyDict[ action ];
 	if ( !pEntry ) {
-		QString str;
-		str.sprintf(
-			"KGlobalAccel : Cannot connect action %s"\
-			"which is not in the object dictionary", action );
-		warning( str );
+		QString str = i18n("KGlobalAccel : Cannot connect action %1 "
+				   "which is not in the object dictionary").arg(action);
+		warning( str.ascii() );
 		return;
 	}
 	
@@ -95,7 +122,7 @@ uint KGlobalAccel::count() const
 	return aKeyDict.count();
 }
 
-uint KGlobalAccel::currentKey( const char * action )
+uint KGlobalAccel::currentKey( const QString& action ) const
 {
 	KKeyEntry *pEntry = aKeyDict[ action ];
 	
@@ -105,7 +132,7 @@ uint KGlobalAccel::currentKey( const char * action )
 		return pEntry->aCurrentKeyCode;
 }
 
-uint KGlobalAccel::defaultKey( const char * action )
+uint KGlobalAccel::defaultKey( const QString& action ) const
 {
 	KKeyEntry *pEntry = aKeyDict[ action ];
 	
@@ -115,16 +142,16 @@ uint KGlobalAccel::defaultKey( const char * action )
         return pEntry->aDefaultKeyCode;
 }
 
-void KGlobalAccel::disconnectItem( const char * action,
-				   const QObject* /*receiver*/, const char* /*member*/ )
+void KGlobalAccel::disconnectItem( const QString& action,
+				   const QObject* /*receiver*/, const char */*member*/ )
 {
     KKeyEntry *pEntry = aKeyDict[ action ];
     if ( !pEntry )
-		return;
-	
+	return;
+
 }
 
-const char * KGlobalAccel::findKey( int key ) const
+QString KGlobalAccel::findKey( int key ) const
 {
 	QDictIterator<KKeyEntry> aKeyIt( aKeyDict );
 	aKeyIt.toFirst();
@@ -134,17 +161,19 @@ const char * KGlobalAccel::findKey( int key ) const
 		++aKeyIt;
 	}
 #undef pE
-	return 0;	
+	return QString::null;	
 }
 
 bool grabFailed;
 
-static int XGrabErrorHandler( Display *, XErrorEvent *e ) {
+extern "C" {
+  static int XGrabErrorHandler( Display *, XErrorEvent *e ) {
 	if ( e->error_code != BadAccess ) {
 		warning( "grabKey: got X error %d instead of BadAccess", e->type );
 	}
 	grabFailed = true;
 	return 0;
+  }
 }
 
 bool KGlobalAccel::grabKey( uint keysym, uint mod ) {
@@ -200,7 +229,7 @@ bool KGlobalAccel::grabKey( uint keysym, uint mod ) {
 	return true;
 }
 
-bool KGlobalAccel::insertItem(  const char* descr, const char * action, uint keyCode,
+bool KGlobalAccel::insertItem(  const QString& descr, const QString& action, uint keyCode,
 					   bool configurable )
 {
 	KKeyEntry *pEntry = aKeyDict[ action ];
@@ -220,22 +249,22 @@ bool KGlobalAccel::insertItem(  const char* descr, const char * action, uint key
 	pEntry->member = 0;
 	pEntry->descr = descr;
 
-	return TRUE;
+	return true;
 }
 
-bool KGlobalAccel::insertItem( const char* descr, const char * action,
-					   const char * keyCode, bool configurable )
+bool KGlobalAccel::insertItem( const QString& descr, const QString& action,
+					   const QString& keyCode, bool configurable )
 {
-	uint iKeyCode = stringToKey( keyCode );
+	uint iKeyCode = KAccel::stringToKey( keyCode );
 	return insertItem(descr, action, iKeyCode, configurable);
 }
 
-bool KGlobalAccel::isEnabled()
+bool KGlobalAccel::isEnabled() const
 {
 	return bEnabled;
 }
 
-bool KGlobalAccel::isItemEnabled( const char *action )
+bool KGlobalAccel::isItemEnabled( const QString& action ) const
 {
 	KKeyEntry *pEntry = aKeyDict[ action ];
 	
@@ -257,7 +286,7 @@ void KGlobalAccel::readSettings()
 	KConfig globalConfig;// this way we are certain to get the global stuff!
 	KConfig *pConfig = &globalConfig;
 
-	pConfig->setGroup( aGroup.data() );
+	pConfig->setGroup( aGroup );
 
 	QDictIterator<KKeyEntry> aKeyIt( aKeyDict );
 	aKeyIt.toFirst();
@@ -280,7 +309,7 @@ void KGlobalAccel::readSettings()
 		if ( s.isNull() )
 			pE->aConfigKeyCode = pE->aDefaultKeyCode;
 		else
-			pE->aConfigKeyCode = stringToKey( s.data() );
+			pE->aConfigKeyCode = KAccel::stringToKey( s );
 		
 		pE->aCurrentKeyCode = pE->aConfigKeyCode;
 		
@@ -295,7 +324,7 @@ void KGlobalAccel::readSettings()
 #undef pE
 }
 	
-void KGlobalAccel::removeItem( const char * action )
+void KGlobalAccel::removeItem( const QString& action )
 {
     KKeyEntry *pEntry = aKeyDict[ action ];
 	
@@ -308,14 +337,14 @@ void KGlobalAccel::removeItem( const char * action )
 	aKeyDict.remove( action );
 }
 
-void KGlobalAccel::setConfigGroup( const char *group )
+void KGlobalAccel::setConfigGroup( const QString& group )
 {
 	aGroup = group;
 }
 
-const char *KGlobalAccel::configGroup()
+QString KGlobalAccel::configGroup() const
 {
-	return aGroup.data();
+	return aGroup;
 }
 
 void KGlobalAccel::setEnabled( bool activate )
@@ -331,17 +360,15 @@ void KGlobalAccel::setEnabled( bool activate )
 	bEnabled = activate;
 }
 
-void KGlobalAccel::setItemEnabled( const char * action, bool activate )
+void KGlobalAccel::setItemEnabled( const QString& action, bool activate )
 {	
 
     KKeyEntry *pEntry = aKeyDict[ action ];
 	if ( !pEntry ) {
-		QString str;
-		str.sprintf(
-			"KGlobalAccel : cannont enable action %s"\
-			"which is not in the object dictionary", action );
-		warning( str );
-		return;
+	    QString str = i18n("KGlobalAccel : cannot enable action %1 "
+			       "which is not in the object dictionary").arg(action);
+	    warning( str.ascii() );
+	    return;
 	}
 
 	bool old = pEntry->bEnabled;
@@ -472,14 +499,14 @@ void KGlobalAccel::writeSettings()
 	KConfig globalConfig;// this way we are certain to get the global stuff!
 	KConfig *pConfig = &globalConfig;
 
-	pConfig->setGroup( aGroup.data() );
+	pConfig->setGroup( aGroup );
 
 	QDictIterator<KKeyEntry> aKeyIt( aKeyDict );
 	aKeyIt.toFirst();
 	while ( aKeyIt.current() ) {
 	  if ( aKeyIt.current()->bConfigurable ){
 		  pConfig->writeEntry( aKeyIt.currentKey(),
-				       keyToString( aKeyIt.current()->aCurrentKeyCode),
+				       KAccel::keyToString( aKeyIt.current()->aCurrentKeyCode),
 				       true, true);
 	  }
 		++aKeyIt;
@@ -499,7 +526,7 @@ bool KGlobalAccel::x11EventFilter( const XEvent *event_ ) {
 	QDictIterator<KKeyEntry> *aKeyIt = new QDictIterator<KKeyEntry>( aKeyDict );
 	aKeyIt->toFirst();
 #define pE aKeyIt->current()
-	while( pE ) { 
+	while( pE ) {
 		int kc = pE->aCurrentKeyCode;
 		if ( mod == keyToXMod( kc ) && keysym == keyToXSym( kc ) ) {
 		break;
@@ -511,18 +538,17 @@ bool KGlobalAccel::x11EventFilter( const XEvent *event_ ) {
 		return false;
 	}
 	
-	if ( !pE ) {
-		return false;
-	}
-
 
 	XAllowEvents(qt_xdisplay(), AsyncKeyboard, CurrentTime);
 	XUngrabKeyboard(qt_xdisplay(), CurrentTime);
 	XSync(qt_xdisplay(), false);
-	connect( this, SIGNAL( activated() ), pE->receiver, pE->member);
-	emit activated();
-	disconnect( this, SIGNAL( activated() ), pE->receiver, pE->member );
+	if ( !QWidget::keyboardGrabber() ) {
+	    connect( this, SIGNAL( activated() ), pE->receiver, pE->member);
+	    emit activated();
+	    disconnect( this, SIGNAL( activated() ), pE->receiver, pE->member );
+	}
 
+#undef pE
 	return true;
 }
 
@@ -534,11 +560,11 @@ uint keyToXMod( uint keyCode )
 	
 	if ( keyCode == 0 ) return mod;
 	
-	if ( keyCode & SHIFT )
+	if ( keyCode & Qt::SHIFT )
 		 mod |= ShiftMask;
-	if ( keyCode & CTRL )
+	if ( keyCode & Qt::CTRL )
 		 mod |= ControlMask;
-	if ( keyCode & ALT )
+	if ( keyCode & Qt::ALT )
 		 mod |= Mod1Mask;
 		
 	return mod;
@@ -547,18 +573,17 @@ uint keyToXMod( uint keyCode )
 uint keyToXSym( uint keyCode )
 {
 	char *toks[4], *next_tok;
+        char sKey[100];
 	int nb_toks = 0;
-	char sKey[200];
 
 	uint keysym = 0;
-	QString s = keyToString( keyCode);
-	
-	strncpy(sKey, (const char *)s.data(), 200);
+	QString s = KAccel::keyToString( keyCode);
 	
 	if ( s.isEmpty() ) return keysym;
-	
+
+        qstrncpy(sKey, s.ascii(), sizeof(sKey));
 	next_tok = strtok( sKey, "+" );
-	
+
 	if ( next_tok == 0L ) return 0;
 	
 	do {
@@ -570,14 +595,14 @@ uint keyToXSym( uint keyCode )
 
 	// Test for exactly one key (other tokens are accelerators)
 	// Fill the keycode with infos
-	bool  keyFound = FALSE;
+	bool  keyFound = false;
 	for ( int i=0; i<nb_toks; i++ ) {
-		if ( strcmp( toks[i], "SHIFT" ) != 0 &&
-			 strcmp( toks[i], "CTRL" ) != 0 &&
-			 strcmp( toks[i], "ALT" ) != 0 ) {
+		if (stricmp(toks[i], "SHIFT") != 0 &&
+		    stricmp(toks[i], "CTRL")  != 0 &&
+		    stricmp(toks[i], "ALT")   != 0) {
 		   if ( keyFound ) return 0;
-		   keyFound = TRUE;
-		   QString l = toks[i];
+		   keyFound = true;
+		   QCString l = toks[i];
 		   l = l.lower();
 		   keysym = XStringToKeysym(l.data());
 		   if (keysym == NoSymbol){
