@@ -1,6 +1,6 @@
 /*
 
-   $Id: kprocess.cpp,v 1.29.4.6 1999/08/18 15:23:49 waba Exp $
+   $Id: kprocess.cpp,v 1.47 2000/04/16 22:55:30 waba Exp $
 
    This file is part of the KDE libraries
    Copyright (C) 1997 Christian Czezatke (e9025461@student.tuwien.ac.at)
@@ -19,64 +19,6 @@
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
-
-   $Log: kprocess.cpp,v $
-   Revision 1.29.4.6  1999/08/18 15:23:49  waba
-   WABA: Fix for communication with child processes
-
-   Revision 1.29.4.5  1999/07/22 21:14:17  porten
-   applied Waba's fix for the fix from HEAD
-
-   Revision 1.29.4.4  1999/07/12 10:26:42  porten
-   kprocess deadlock fix from Alex Hayward (bug #1002 and #1513)
-
-   Revision 1.29.4.3  1999/07/04 17:48:09  porten
-   check for Stdin flag in writeStdin
-
-   Revision 1.29.4.2  1999/05/10 19:33:18  gehrmab
-   Fixed memory leak in KProcess
-
-   Revision 1.29.4.1  1999/04/07 15:39:11  porten
-   restore SIGPIPE handler
-
-   Revision 1.29  1999/01/18 10:56:22  kulow
-   .moc files are back in kdelibs. Built fine here using automake 1.3
-
-   Revision 1.28  1999/01/15 09:30:40  kulow
-   it's official - kdelibs builds with srcdir != builddir. For this I
-   automocifized it, the generated rules are easier to maintain than
-   selfwritten rules. I have to fight with some bugs of this tool, but
-   generally it's better than keeping them updated by hand.
-
-   Revision 1.27  1998/10/28 19:00:15  ettrich
-   small fix (thanks to Christian Stoeckl)
-
-   Revision 1.26  1998/10/19 08:09:05  ettrich
-   small pseudo fix to kshellprocess reverted
-
-   Revision 1.25  1998/09/22 18:03:13  ettrich
-   Matthias: small updates to kaccel
-
-   Revision 1.24  1998/09/17 19:22:09  radej
-   sven: made it work with egcs-1.1
-
-   Revision 1.23  1998/09/01 20:21:29  kulow
-   I renamed all old qt header files to the new versions. I think, this looks
-   nicer (and gives the change in configure a sense :)
-
-   Revision 1.22  1998/07/29 10:14:28  kulow
-   porting to a virtual plattform called "tajsandmineansiplatform" :)
-
-   Revision 1.21  1998/07/29 09:07:50  ssk
-   Fixed a whole lot of -Wall -ansi -pedantic warnings.
-
-   Revision 1.20  1998/03/10 18:59:22  mario
-   Mario: fixed a memory leak in KShellProcess (shell not freed)
-
-   Revision 1.19  1998/03/08 17:21:40  wuebben
-   Bernd: Fixed the segfault problem in 'KShellProcess::start()'.
-
-
 */
 
 
@@ -100,22 +42,30 @@
 #define _MAY_INCLUDE_KPROCESSCONTROLLER_
 #include "kprocctrl.h"
 
-#include <qapplication.h>
-
-// to define kstrdup
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
 
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include <string.h>
-#include <errno.h>
 #include <sys/socket.h>
+
+#include <errno.h>
 #include <fcntl.h>
+#include <stdlib.h>
 #include <signal.h>
 #include <stdio.h>
+#include <string.h>
+#include <unistd.h>
+#ifdef HAVE_VFORK_H
+#include <vfork.h>
+#endif
+#ifdef HAVE_SYS_SELECT_H
+#include <sys/select.h>
+#endif
+
+#include <qapplication.h>
 
 /////////////////////////////
 // public member functions //
@@ -124,13 +74,15 @@
 
 KProcess::KProcess()
 {
-  if (0 == theKProcessController) {
-	theKProcessController= new KProcessController();
-	CHECK_PTR(theKProcessController);
+  arguments.setAutoDelete(true);
+
+  if (0 == KProcessController::theKProcessController) {
+	KProcessController::theKProcessController= new KProcessController();
+	CHECK_PTR(KProcessController::theKProcessController);
   }
 
   run_mode = NotifyOnExit;
-  runs = FALSE;
+  runs = false;
   pid = 0;
   status = 0;
   innot = outnot = errnot = 0;
@@ -139,7 +91,7 @@ KProcess::KProcess()
   input_sent = 0;
   input_total = 0;
 
-  theKProcessController->processList->append(this);
+  KProcessController::theKProcessController->processList->append(this);
 }
 
 
@@ -151,7 +103,7 @@ KProcess::~KProcess()
   // list of valid processes (if the process is not started as
   // "DontCare")
 
-  theKProcessController->processList->remove(this);
+  KProcessController::theKProcessController->processList->remove(this);
   // this must happen before we kill the child
   // TODO: block the signal while removing the current process from the process list
 
@@ -163,16 +115,21 @@ KProcess::~KProcess()
 
 
 
-bool KProcess::setExecutable(const char *proc)
+bool KProcess::setExecutable(const QString& proc)
 {
-  if (runs) return FALSE;
+  char *hlp;
+
+
+  if (runs) return false;
 
   arguments.removeFirst();
   if (0 != proc) {
-    arguments.insert(0,proc);
+    hlp = qstrdup(proc.ascii());
+    CHECK_PTR(hlp);
+    arguments.insert(0,hlp);
   }
 
-  return TRUE;
+  return true;
 }
 
 
@@ -180,10 +137,12 @@ bool KProcess::setExecutable(const char *proc)
 
 
 
-KProcess &KProcess::operator<<(const char *arg)
+KProcess &KProcess::operator<<(const QString& arg)
 {
-  CHECK_PTR(arg);
-  arguments.append(arg);
+  char *new_arg= qstrdup(arg.ascii());
+
+  CHECK_PTR(new_arg);
+  arguments.append(new_arg);
   return *this;
 }
 
@@ -206,7 +165,7 @@ bool KProcess::start(RunMode runmode, Communication comm)
   char **arglist;
 
   if (runs || (0 == n)) {
-	return FALSE;  // cannot start a process that is already running
+	return false;  // cannot start a process that is already running
 	// or if no executable has been assigned
   }
   run_mode = runmode;
@@ -221,8 +180,10 @@ bool KProcess::start(RunMode runmode, Communication comm)
   if (!setupCommunication(comm))
     debug("Could not setup Communication!");
 
-  runs = TRUE;
-  pid = fork();
+  runs = true;
+
+  QApplication::flushX();
+  pid = vfork();
 
   if (0 == pid) {
 	// The child process
@@ -242,14 +203,14 @@ bool KProcess::start(RunMode runmode, Communication comm)
         sigaction(SIGPIPE, &act, 0L);
 
 	execvp(arglist[0], arglist);
-	exit(-1);
+	_exit(-1);
 
   } else if (-1 == pid) {
 	// forking failed
 
-	runs = FALSE;
+	runs = false;
 	free(arglist);
-	return FALSE;
+	return false;
 
   } else {
 	// the parent continues here
@@ -271,14 +232,14 @@ bool KProcess::start(RunMode runmode, Communication comm)
 	}
   }
   free(arglist);
-  return TRUE;
+  return true;
 }
 
 
 
 bool KProcess::kill(int signo)
 {
-  bool rv=FALSE;
+  bool rv=false;
 
   if (0 != pid)
     rv= (-1 != ::kill(pid, signo));
@@ -318,7 +279,7 @@ int KProcess::exitStatus()
 
 
 
-bool KProcess::writeStdin(char *buffer, int buflen)
+bool KProcess::writeStdin(const char *buffer, int buflen)
 {
   bool rv;
 
@@ -326,21 +287,31 @@ bool KProcess::writeStdin(char *buffer, int buflen)
   // to stdout is not allowed (since it could also confuse
   // kprocess...
   if (0 != input_data)
-    return FALSE;
+    return false;
 
   if (runs && (communication & Stdin)) {
     input_data = buffer;
     input_sent = 0;
     input_total = buflen;
     slotSendData(0);
-    innot->setEnabled(TRUE);
-    rv = TRUE;
+    innot->setEnabled(true);
+    rv = true;
   } else
-    rv = FALSE;
+    rv = false;
   return rv;
 }
 
+void KProcess::suspend()
+{
+  if ((communication & Stdout) && outnot)
+     outnot->setEnabled(false);
+}
 
+void KProcess::resume()
+{
+  if ((communication & Stdout) && outnot)
+     outnot->setEnabled(true);
+}
 
 bool KProcess::closeStdin()
 {
@@ -348,11 +319,42 @@ bool KProcess::closeStdin()
 
   if (communication & Stdin) {
     communication = (Communication) (communication & ~Stdin);
-    innot->setEnabled(FALSE);
+    delete innot;
+    innot = 0;
     close(in[1]);
-    rv = TRUE;
+    rv = true;
   } else
-    rv = FALSE;
+    rv = false;
+  return rv;
+}
+
+bool KProcess::closeStdout()
+{
+  bool rv;
+
+  if (communication & Stdout) {
+    communication = (Communication) (communication & ~Stdout);
+    delete outnot;
+    outnot = 0;
+    close(out[0]);
+    rv = true;
+  } else
+    rv = false;
+  return rv;
+}
+
+bool KProcess::closeStderr()
+{
+  bool rv;
+
+  if (communication & Stderr) {
+    communication = (Communication) (communication & ~Stderr);
+    delete errnot;
+    errnot = 0;
+    close(err[0]);
+    rv = true;
+  } else
+    rv = false;
   return rv;
 }
 
@@ -365,26 +367,22 @@ bool KProcess::closeStdin()
 
 void KProcess::slotChildOutput(int fdno)
 {
-  if (!childOutput(fdno)) {
-	outnot->setEnabled(FALSE);	
-  }
+  if (!childOutput(fdno)) 
+     closeStdout();	
 }
 
 
-
 void KProcess::slotChildError(int fdno)
 {
-  if (!childError(fdno)) {
-    errnot->setEnabled(FALSE);
-  }
+  if (!childError(fdno))
+     closeStderr();
 }
 
 
-
 void KProcess::slotSendData(int)
 {
   if (input_sent == input_total) {
-    innot->setEnabled(FALSE);
+    innot->setEnabled(false);
     input_data = 0;
     emit wroteStdin(this);
   } else
@@ -401,7 +399,7 @@ void KProcess::slotSendData(int)
 
 void KProcess::processHasExited(int state)
 {
-  runs = FALSE;
+  runs = false;
   status = state;
 
   commClose(); // cleanup communication sockets
@@ -415,15 +413,24 @@ void KProcess::processHasExited(int state)
 
 int KProcess::childOutput(int fdno)
 {
-  char buffer[1024];
-  int len;
+  if (communication & NoRead) {
+     int len = -1;
+     emit receivedStdout(fdno, len);
+     errno = 0; // Make sure errno doesn't read "EAGAIN"
+     return len;
+  }
+  else
+  {
+     char buffer[1024];
+     int len;
 
-  len = ::read(fdno, buffer, 1024);
+     len = ::read(fdno, buffer, 1024);
 
-  if ( 0 < len) {
-	emit receivedStdout(this, buffer, len);
+     if ( 0 < len) {
+   	emit receivedStdout(this, buffer, len);
+     }
+     return len;
   }
-  return len;
 }
 
 
@@ -480,24 +487,26 @@ int KProcess::commSetupDoneP()
 	if (run_mode == Block) return ok;
 
 	if (communication & Stdin) {
-	  ok &= (-1 != fcntl(in[1], F_SETFL, O_NONBLOCK));
+//	  ok &= (-1 != fcntl(in[1], F_SETFL, O_NONBLOCK));
 	  innot =  new QSocketNotifier(in[1], QSocketNotifier::Write, this);
 	  CHECK_PTR(innot);
-	  innot->setEnabled(FALSE); // will be enabled when data has to be sent
+	  innot->setEnabled(false); // will be enabled when data has to be sent
 	  QObject::connect(innot, SIGNAL(activated(int)),
 					   this, SLOT(slotSendData(int)));
 	}
 
 	if (communication & Stdout) {
-	  ok &= (-1 != fcntl(out[0], F_SETFL, O_NONBLOCK));
+//	  ok &= (-1 != fcntl(out[0], F_SETFL, O_NONBLOCK));
 	  outnot = new QSocketNotifier(out[0], QSocketNotifier::Read, this);
 	  CHECK_PTR(outnot);
 	  QObject::connect(outnot, SIGNAL(activated(int)),
 					   this, SLOT(slotChildOutput(int)));
+          if (communication & NoRead)
+              suspend();
 	}
 
 	if (communication & Stderr) {
-	  ok &= (-1 != fcntl(err[0], F_SETFL, O_NONBLOCK));
+//	  ok &= (-1 != fcntl(err[0], F_SETFL, O_NONBLOCK));
 	  errnot = new QSocketNotifier(err[0], QSocketNotifier::Read, this );
 	  CHECK_PTR(errnot);
 	  QObject::connect(errnot, SIGNAL(activated(int)),
@@ -560,18 +569,22 @@ void KProcess::commClose()
 
           int max_fd = 0;
           if (b_out) {
+	    fcntl(out[0], F_SETFL, O_NONBLOCK);
             if (out[0] > max_fd)
               max_fd = out[0];
             delete outnot;
+            outnot = 0;
           }
           if (b_err) {
+	    fcntl(err[0], F_SETFL, O_NONBLOCK);
             if (err[0] > max_fd)
               max_fd = err[0];
             delete errnot;
+            errnot = 0;
           }
            
 
-	  while (b_out || b_err) {
+	  while (1) {
 	    FD_ZERO(&rfds);
             if (b_out) 
 	      FD_SET(out[0], &rfds);
@@ -583,24 +596,24 @@ void KProcess::commClose()
 	    if (fds_ready <= 0) break;
 
 	    if (b_out && FD_ISSET(out[0], &rfds)) {
-	      int ret = childOutput(out[0]);
-	      if ((ret == -1 && errno != EAGAIN) || ret == 0) 
-		b_out = false;
+	      int ret = 1;
+	      while (ret > 0) ret = childOutput(out[0]);
+	      if ((ret == -1 && errno != EAGAIN) || ret == 0) break;
 	    }
                                
 	    if (b_err && FD_ISSET(err[0], &rfds)) {
-	      int ret = childError(err[0]);
-	      if ((ret == -1 && errno != EAGAIN) || ret == 0) 
-		b_err = false;
+	      int ret = 1;
+	      while (ret > 0) ret = childError(err[0]);
+	      if ((ret == -1 && errno != EAGAIN) || ret == 0) break;
 	    }
 	  }
 	}
 
-	if (communication & Stdin)
+	if (b_in)
 	    close(in[1]);
-	if (communication & Stdout)
+	if (b_out)
 	    close(out[0]);
-	if (communication & Stderr)
+	if (b_err)
 	    close(err[0]);
 	
 	communication = NoCommunication;
@@ -618,26 +631,25 @@ KShellProcess::KShellProcess(const char *shellname):
   KProcess()
 {
   if (0 != shellname)
-    shell = kstrdup(shellname);
+    shell = qstrdup(shellname);
   else
     shell = 0;
 }
 
 
 KShellProcess::~KShellProcess() {
-  if(shell)
-    free(shell);
+  delete [] shell;
 }
 
 bool KShellProcess::start(RunMode runmode, Communication comm)
 {
   uint i;
   uint n = arguments.count();
-  char *arglist[4];
+  const char *arglist[4];
   QString cmd;
 
   if (runs || (0 == n)) {
-	return FALSE;  // cannot start a process that is already running
+	return false;  // cannot start a process that is already running
 	// or if no executable has been assigned
   }
 
@@ -648,7 +660,7 @@ bool KShellProcess::start(RunMode runmode, Communication comm)
     shell = searchShell();
   if (0 == shell) {
     debug("Could not find a valid shell\n");
-    return FALSE;
+    return false;
   }
 
   // CC: Changed the way the parameter was built up
@@ -667,14 +679,16 @@ bool KShellProcess::start(RunMode runmode, Communication comm)
 //   cmd.stripWhiteSpace();
 //   if (cmd[cmd.length()-1] != '&')
 //       cmd += '&';
-  arglist[2] = cmd.data();
+  arglist[2] = qstrdup(cmd.ascii());
   arglist[3] = 0;
 
   if (!setupCommunication(comm))
     debug("Could not setup Communication!");
 
-  runs = TRUE;
-  pid = fork();
+  runs = true;
+
+  QApplication::flushX();	
+  pid = vfork();
 
   if (0 == pid) {
 	// The child process
@@ -694,14 +708,15 @@ bool KShellProcess::start(RunMode runmode, Communication comm)
         act.sa_flags = 0;
         sigaction(SIGPIPE, &act, 0L);
 
-	execvp(arglist[0], arglist);
-	exit(-1);
+	execvp(arglist[0], const_cast<char *const *>(arglist));
+	_exit(-1);
 
   } else if (-1 == pid) {
 	// forking failed
 
-	runs = FALSE;
-	return FALSE;
+	runs = false;
+	//	free(arglist);
+	return false;
 
   } else {
 	// the parent continues here
@@ -723,7 +738,8 @@ bool KShellProcess::start(RunMode runmode, Communication comm)
 	  emit processExited(this);
 	}
   }
-  return TRUE;
+  //  free(arglist);
+  return true;
 }
 
 
@@ -737,7 +753,7 @@ char *KShellProcess::searchShell()
   // CC: now get the name of the shell we have to use
   hlp = getenv("SHELL");
   if (isExecutable(hlp)) {
-    copy = kstrdup(hlp);
+    copy = qstrdup(hlp);
     CHECK_PTR(copy);
   }
 
@@ -745,8 +761,8 @@ char *KShellProcess::searchShell()
     // CC: hmm, invalid $SHELL in environment -- maybe there are whitespaces to be stripped?
     QString stmp = QString(shell);
     QString shell_stripped = stmp.stripWhiteSpace();
-    if (isExecutable(shell_stripped.data())) {
-      copy = kstrdup(shell_stripped.data());
+    if (isExecutable(shell_stripped.ascii())) {
+      copy = qstrdup(shell_stripped.ascii());
       CHECK_PTR(copy);
     }
   }
@@ -760,12 +776,12 @@ bool KShellProcess::isExecutable(const char *fname)
 {
   struct stat fileinfo;
 
-  if ((0 == fname) || (strlen(fname) == 0)) return FALSE;
+  if ((0 == fname) || (strlen(fname) == 0)) return false;
   // CC: filename is invalid
 
   // CC: we've got a valid filename, now let's see whether we can execute that file
 
-  if (-1 == stat(fname, &fileinfo)) return FALSE;
+  if (-1 == stat(fname, &fileinfo)) return false;
   // CC: return false if the file does not exist
 
   // CC: anyway, we cannot execute directories, block/character devices, fifos or sockets
@@ -778,14 +794,14 @@ bool KShellProcess::isExecutable(const char *fname)
 #endif
        (S_ISFIFO(fileinfo.st_mode)) ||
        (S_ISDIR(fileinfo.st_mode)) ) {
-    return FALSE;
+    return false;
   }
 
   // CC: now check for permission to execute the file
-  if (access(fname, X_OK) != 0) return FALSE;
+  if (access(fname, X_OK) != 0) return false;
 
   // CC: we've passed all the tests...
-  return TRUE;
+  return true;
 }
 #include "kprocess.moc"
 
