@@ -1,216 +1,127 @@
-/* This file is part of the KDE libraries
-    Copyright (C) 1997 Torben Weis (weis@kde.org)
-
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
-*/
 /*
- * $Id: ksock.cpp,v 1.24.4.5 1999/08/23 09:16:04 waba Exp $
- * $Log: ksock.cpp,v $
- * Revision 1.24.4.5  1999/08/23 09:16:04  waba
- * WABA: Don't kill the application upon problems.
+ *  This file is part of the KDE libraries
+ *  Copyright (C) 1997 Torben Weis (weis@kde.org)
  *
- * Revision 1.24.4.4  1999/07/09 21:16:41  porten
- * patch for bug #1336,1549 from Ingo <schneidi@informatik.tu-muenchen.de>
+ *  $Id: ksock.cpp,v 1.46 2000/04/19 06:55:25 waba Exp $
  *
- * Revision 1.24.4.3  1999/04/18 19:49:23  kulow
- * fixing bug reported by litsch.iep@t-online.de (Stephan Litsch)
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2 of the License, or (at your option) any later version.
  *
- * Revision 1.24.4.2  1999/04/01 20:43:44  pbrown
- * socket patch from Dirk A. Mueller <dmuell@gmx.net>, forwarded to kde-devel
- * by Torben, applied.
+ *  This library is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Library General Public License for more details.
  *
- * Revision 1.24.4.1  1999/02/24 12:49:17  dfaure
- * getdtablesize() -> getrlimit(). Fixes #447 and removes a #ifdef HPUX.
- *
- * Revision 1.25  1999/02/24 12:47:34  dfaure
- * getdtablesize() -> getrlimit(). Fixes #447 and removes a #ifdef HPUX.
- *
- * Revision 1.24  1999/01/18 10:56:25  kulow
- * .moc files are back in kdelibs. Built fine here using automake 1.3
- *
- * Revision 1.23  1999/01/15 09:30:42  kulow
- * it's official - kdelibs builds with srcdir != builddir. For this I
- * automocifized it, the generated rules are easier to maintain than
- * selfwritten rules. I have to fight with some bugs of this tool, but
- * generally it's better than keeping them updated by hand.
- *
- * Revision 1.22  1999/01/11 23:09:51  thufir
- * fix: fixes bug where it would loop n - tries, after a successful connect (would cause a pause after successful connect)
- *
- * Revision 1.21  1998/11/11 00:02:54  thufir
- * addes ability to set a connect Time Out, breaks binary compatability
- *
- * Revision 1.20  1998/08/02 14:49:31  kalle
- * ANother try at the socket problem. Hope that this works on _all_ platforms now.
- *
- * Revision 1.19  1998/07/29 12:39:17  kalle
- * Don't hardcode maximum of pending connections in listen(). Should work on all platforms including QNX.
- *
- * Revision 1.18  1998/02/20 02:37:25  torben
- * Torben: Fixes permissions
- *
- * Revision 1.17  1998/01/23 11:25:28  kulow
- * Solaris doesn't define UNIX_PATH_MAX
- *
- * Revision 1.16  1998/01/23 02:23:38  torben
- * Torben: Supports UNIX domain sockets now.
- *
- * Revision 1.15  1998/01/18 14:39:03  kulow
- * reverted the changes, Jacek commited.
- * Only the RCS comments were affected, but to keep them consistent, I
- * thought, it's better to revert them.
- * I checked twice, that only comments are affected ;)
- *
- * Revision 1.13  1997/12/18 01:56:24  torben
- * Torben: Secure string operations. Use instead of QString::sprintf
- *
- * Revision 1.12  1997/11/29 17:58:48  kalle
- * Alpha patches
- *
- * Revision 1.11  1997/11/09 01:52:47  torben
- * Torben: Fixed port number bug
- *
- * Revision 1.10  1997/10/21 20:44:52  kulow
- * removed all NULLs and replaced it with 0L or "".
- * There are some left in mediatool, but this is not C++
- *
- * Revision 1.9  1997/10/16 11:15:03  torben
- * Kalle: Copyright headers
- * kdoctoolbar removed
- *
- * Revision 1.8  1997/09/18 12:16:04  kulow
- * corrected some header dependencies. Removed most of them in drag.h and put
- * them in drag.cpp. Now it should compile even under SunOS 4.4.1 ;)
- *
- * Revision 1.7  1997/08/30 08:32:54  kdecvs
- * Coolo: changed the location of the include files to get rid of the
- * hardcoded HAVE_STDC_HEADERS
- *
- * Revision 1.6  1997/07/27 13:43:58  kalle
- * Even more SGI and SCC patches, security patch for kapp, various fixes for ksock
- *
- * Revision 1.5  1997/07/17 18:43:18  kalle
- * Kalle: new stopsign.xpm
- * 			KConfig: environment variables are resolved in readEntry()
- * 			(meaning you can write() an entry with an environment
- * 			variable and read it back in, and the value gets properly
- * 			expanded).
- * 			Fixed three bugs in ksock.cpp
- * 			Added KFloater
- * 			Added KCombo
- * 			Added KLineEdit
- * 			New KToolbar
- * 			New KToplevelWidget
- *
- * Revision 1.4  1997/05/30 20:04:38  kalle
- * Kalle:
- * 30.05.97:	signal handler for reaping zombie help processes reinstalls itself
- * 		patch to KIconLoader by Christian Esken
- * 		slightly better look for KTabCtl
- * 		kdecore Makefile does not expect current dir to be in path
- * 		Better Alpha support
- *
- * Revision 1.3  1997/05/09 15:10:11  kulow
- * Coolo: patched ltconfig for FreeBSD
- * removed some stupid warnings
- *
- * Revision 1.2  1997/04/28 06:57:46  kalle
- * Various widgets moved from apps to libs
- * Added KSeparator
- * Several bugs fixed
- * Patches from Matthias Ettrich
- * Made ksock.cpp more alpha-friendly
- * Removed XPM-Loading from KPixmap
- * Reaping zombie KDEHelp childs
- * WidgetStyle of KApplication objects configurable via kdisplay
- */
-
-#include <qapplication.h>
+ *  You should have received a copy of the GNU Library General Public License
+ *  along with this library; see the file COPYING.LIB.  If not, write to
+ *  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ *  Boston, MA 02111-1307, USA.
+ **/
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#include <stdio.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+// on Linux/libc5, this includes linux/socket.h where SOMAXCONN is defined
+#include <sys/socket.h>
+#include <sys/resource.h>
+#include <sys/time.h>
+#include <sys/un.h>
+#ifdef HAVE_SYS_SELECT_H
+#include <sys/select.h>
+#endif
+#include <netinet/in.h>
+
+#include <arpa/inet.h>
+
+#include "ksock.h"
+
 #include <errno.h>
-#ifdef STDC_HEADERS
+#include <fcntl.h>
+#include <netdb.h>
+// defines MAXDNAME under Solaris
+#include <arpa/nameser.h>
+#include <resolv.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#endif
-#ifdef HAVE_UNISTD_H
+#include <signal.h>
 #include <unistd.h>
-#endif
+
 #ifdef HAVE_SYSENT_H
 #include <sysent.h>
 #endif
-#include <sys/types.h>
-#include <sys/stat.h>
+
 #if TIME_WITH_SYS_TIME
-# include <sys/time.h>
-# include <time.h>
-#else
-# if HAVE_SYS_TIME_H
-#  include <sys/time.h>
-# else
-#  include <time.h>
-# endif
-#endif
-#ifdef HAVE_SYS_SOCKET_H
-// on Linux/libc5, this includes linux/socket.h where SOMAXCONN is defined
-#include <sys/socket.h>
+#include <time.h>
 #endif
+
 // Play it safe, use a reasonable default, if SOMAXCONN was nowhere defined.
 #ifndef SOMAXCONN
 #warning Your header files do not seem to support SOMAXCONN
 #define SOMAXCONN 5
 #endif
 
-#include <sys/resource.h>
-#include <sys/time.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <netdb.h>
-#include <fcntl.h>
-#include <signal.h>
-
-#include "ksock.h"
+#include <qapplication.h>
+#include <qsocketnotifier.h>
 
 #ifndef UNIX_PATH_MAX
 #define UNIX_PATH_MAX 108 // this is the value, I found under Linux
 #endif
 
+char *KSocket::cachedHostname = 0;
+ksockaddr_in *KSocket::cachedServerName = 0;
+
+// I moved this into here so we could accurately detect the domain, for
+// posterity.  Really.
+KSocket::KSocket( int _sock)
+ : sock(_sock), readNotifier(0), writeNotifier(0)
+{
+  struct sockaddr_in sin;
+  ksize_t len = sizeof(sin);
+
+  memset(&sin, 0, len);
+
+  // getsockname will fill in all the appropiate details, and
+  // since sockaddr_in will exist everywhere and is somewhat compatible
+  // with sockaddr_in6, we can use it to avoid needless ifdefs.
+  getsockname(_sock, (struct sockaddr *)&sin, &len);
+
+  // Now that we've got the domain, remember it
+  domain = sin.sin_family;
+}
+
 KSocket::KSocket( const char *_host, unsigned short int _port, int _timeout ) :
   sock( -1 ), readNotifier( 0L ), writeNotifier( 0L )
 {
     timeOut = _timeout;
-    domain = PF_INET;
-    connect( _host, _port );
-}
+#ifdef INET6
+    // Setup the resolver to look for IPv6 addresses and then
+    // as a fall back, look for IPv4.  This means we don't need further
+    // ifdefs to use gethostbyname2. (Taken from rfc2133)
+    res_init();
+    _res.options |= RES_USE_INET6;
 
-KSocket::KSocket( const char *_host, unsigned short int _port ) :
-  sock( -1 ), readNotifier( 0L ), writeNotifier( 0L )
-{
-    timeOut = 30;
+    // Also we do this because gethostbyname will return IPv4 mapped on to
+    // IPv6 addresses when set like this and use 16 byte hostents, so we
+    // should be safe and use sockaddr_in6, etc, and by setting the domain to
+    // PF_INET6, anyone who checks our sockaddr stuff will avoid buffer
+    // overflows by seeing that it's new style.
+    domain = PF_INET6;
+#else
     domain = PF_INET;
+#endif
     connect( _host, _port );
 }
 
 KSocket::KSocket( const char *_path ) :
   sock( -1 ), readNotifier( 0L ), writeNotifier( 0L )
 {
+  timeOut = 0; // Not used
   domain = PF_UNIX;
   connect( _path );
 }
@@ -269,24 +180,9 @@ void KSocket::slotWrite( int )
  * before connecting to any other socket. Here you must specify the
  * host and port you want to connect to.
  */
-bool KSocket::init_sockaddr( const char *hostname, unsigned short int port )
+bool KSocket::init_sockaddr( const QString& hostname, unsigned short int port )
 {
-  if ( domain != PF_INET )
-    return false;
-  
-  struct hostent *hostinfo;
-  server_name.sin_family = AF_INET;
-  server_name.sin_port = htons( port );
-  hostinfo = gethostbyname( hostname );
-  
-  if ( !hostinfo )
-    {
-	  warning("Unknown host %s.\n",hostname);
-	  return false;	
-    }
-  server_name.sin_addr = *(struct in_addr*) hostinfo->h_addr;    
-  
-  return true;
+  return initSockaddr(&server_name, hostname.ascii(), port, domain);
 }
 
 /*
@@ -297,6 +193,10 @@ bool KSocket::connect( const char *_path )
   if ( domain != PF_UNIX )
     fatal( "Connecting a PF_INET socket to a PF_UNIX domain socket\n");
   
+  sock = ::socket(PF_UNIX,SOCK_STREAM,0);
+  if (sock < 0)
+	return false;
+  
   unix_addr.sun_family = AF_UNIX;
   int l = strlen( _path );
   if ( l > UNIX_PATH_MAX - 1 )
@@ -306,10 +206,6 @@ bool KSocket::connect( const char *_path )
   }  
   strcpy( unix_addr.sun_path, _path );
 
-  sock = ::socket(PF_UNIX,SOCK_STREAM,0);
-  if (sock < 0)
-	return false;
-  
   if ( 0 > ::connect( sock, (struct sockaddr*)(&unix_addr), 
 					  sizeof( unix_addr ) ) )
   {
@@ -324,66 +220,65 @@ bool KSocket::connect( const char *_path )
 /*
  * Connects the socket to _host, _port.
  */
-bool KSocket::connect( const char *_host, unsigned short int _port )
+bool KSocket::connect( const QString& _host, unsigned short int _port )
 {
+#ifdef INET6
+  if ( (domain != PF_INET6) && (domain != PF_INET) )
+#else
   if ( domain != PF_INET )
+#endif
     fatal( "Connecting a PF_UNIX domain socket to a PF_INET domain socket\n");
 
-  sock = ::socket(PF_INET,SOCK_STREAM,0);
+  sock = ::socket(domain, SOCK_STREAM, 0);
   if (sock < 0)
 	return false;
   
-  if ( !init_sockaddr( _host, _port) )
-	{
+  if ( !init_sockaddr( _host, _port) ) {
 	  ::close( sock );
 	  sock = -1;
 	  return false;
-	}
+  }
 
-  fcntl(sock,F_SETFL,(fcntl(sock,F_GETFL)|O_NDELAY));
+  fcntl(sock, F_SETFL, ( fcntl(sock,F_GETFL)|O_NDELAY) );
 
   errno = 0;
   if (::connect(sock, (struct sockaddr*)(&server_name), sizeof(server_name))){
-      if(errno != EINPROGRESS && errno != EWOULDBLOCK){
+      if(errno != EINPROGRESS && errno != EWOULDBLOCK) {
           ::close( sock );
           sock = -1;
           return false;
       }
-  }else
+  } else
       return true;
 
   fd_set rd, wr;
   struct timeval timeout;
-  int ret = 0, n;
 
-  n = timeOut;
-  FD_ZERO(&rd);
-  FD_ZERO(&wr);
-  FD_SET(sock, &rd);
-  FD_SET(sock, &wr);
-//  printf("timeout=%d\n", n);
+  int n = timeOut*10; // Timeout in 1/10th's of a second
   while(n--){
-      timeout.tv_usec = 0;
-      timeout.tv_sec = 1;
+      FD_ZERO(&rd);
+      FD_ZERO(&wr);
+      FD_SET(sock, &rd);
+      FD_SET(sock, &wr);
 
-      struct rlimit rlp;
-      getrlimit(RLIMIT_NOFILE, &rlp); // getdtablesize() equivalent. David Faure.
+      timeout.tv_usec = 100*1000; // 1/10th sec
+      timeout.tv_sec = 0;
 
-      ret = select(rlp.rlim_cur, (fd_set *)&rd, (fd_set *)&wr, (fd_set *)0,
-                   (struct timeval *)&timeout);
-      // if(ret)
-      //    return(true);
+      select(sock + 1, &rd, &wr, (fd_set *)0, &timeout);
 
-      switch (ret)
+      if (FD_ISSET(sock, &rd) || FD_ISSET(sock, &wr))
       {
-	  case 0: break; // Timeout
-	  case 1: case 2: return(true); // Success
-	  default: // Error
-	      ::close(sock);
-	      sock = -1;
-	      return false;
+         int errcode;
+         ksize_t len = sizeof(errcode);
+         int ret = getsockopt(sock, SOL_SOCKET, SO_ERROR, (char*)&errcode, &len);
+         if ((ret == -1) || (errcode != 0))
+         {
+            ::close(sock);
+            sock = -1;
+            return false;
+         }
+         return true;
       }
-
       qApp->processEvents();
       qApp->flushX();
   }
@@ -393,14 +288,69 @@ bool KSocket::connect( const char *_host, unsigned short int _port )
   return false;
 }
 
-unsigned long KSocket::getAddr()
+unsigned long KSocket::ipv4_addr()
 {
   if ( domain != PF_INET )
     return 0;
   
-  struct sockaddr_in name; ksize_t len = sizeof(name);
+  sockaddr_in name; ksize_t len = sizeof(name);
   getsockname(sock, (struct sockaddr *) &name, &len);
-  return ntohl(name.sin_addr.s_addr);
+  if (name.sin_family == AF_INET) // It's IPv4
+    return ntohl(name.sin_addr.s_addr);
+#ifdef INET6
+  else if (name.sin_family == AF_INET6) // It's IPv6 Ah.
+    return 0;
+#endif
+  else // We dunno what it is
+    return 0;
+}
+
+bool KSocket::initSockaddr (ksockaddr_in *server_name, const char *hostname, unsigned short int port, int domain)
+{
+  if (
+#ifdef INET6
+     ( domain != PF_INET6 ) &&
+#endif
+     ( domain != PF_INET ) )
+    return false;  
+
+  if (cachedHostname && (strcmp(hostname, cachedHostname) == 0))
+  {
+     memcpy( server_name, cachedServerName, sizeof(ksockaddr_in));
+     return true;
+  }
+
+  memset(server_name, 0, sizeof(ksockaddr_in));
+
+  struct hostent *hostinfo;
+  hostinfo = gethostbyname( hostname );
+
+  if ( hostinfo == 0L )
+    return false;
+
+#ifdef INET6
+  if (domain == PF_INET6) {
+    server_name->sin6_family = hostinfo->h_addrtype;
+    server_name->sin6_port = htons( port );
+    memcpy(&server_name->sin6_addr, hostinfo->h_addr_list[0], hostinfo->h_length);
+  } else
+#endif
+  {
+    get_sin_pfamily(server_name) = hostinfo->h_addrtype;
+    get_sin_pport(server_name) = htons( port );
+    memcpy(&get_sin_paddr(server_name), hostinfo->h_addr_list[0], hostinfo->h_length);
+  }
+
+  // update our primitive cache 
+  delete [] cachedHostname;
+  cachedHostname = new char[ strlen(hostname)+1 ];
+  strcpy(cachedHostname, hostname);
+
+  if (!cachedServerName)
+     cachedServerName = new ksockaddr_in;
+  memcpy( cachedServerName, server_name, sizeof(ksockaddr_in));
+
+  return true;
 }
 
 KSocket::~KSocket()
@@ -417,13 +367,13 @@ KSocket::~KSocket()
 
 
 KServerSocket::KServerSocket( const char *_path ) :
-  notifier( 0L), sock( -1 )
+  notifier( 0L ), sock( -1 )
 {
   domain = PF_UNIX;
   
   if ( !init ( _path ) )
   {
-    // fatal("Error constructing PF_UNIX domain server socket\n");
+    fatal("Error constructing PF_UNIX domain server socket\n");
     return;
   }
     
@@ -431,7 +381,7 @@ KServerSocket::KServerSocket( const char *_path ) :
   connect( notifier, SIGNAL( activated(int) ), this, SLOT( slotAccept(int) ) );
 }
 
-KServerSocket::KServerSocket( int _port ) :
+KServerSocket::KServerSocket( unsigned short int _port ) :
   notifier( 0L ), sock( -1 )
 {
   domain = PF_INET;
@@ -465,7 +415,7 @@ bool KServerSocket::init( const char *_path )
     return false;
   }
 
-  unlink(_path);   
+  unlink(_path );   
 
   struct sockaddr_un name;
   name.sun_family = AF_UNIX;
@@ -500,30 +450,53 @@ bool KServerSocket::init( const char *_path )
 
 bool KServerSocket::init( unsigned short int _port )
 {
-  if ( domain != PF_INET )
+  if ( 
+#ifdef INET6
+      ( domain != PF_INET6 ) &&
+#endif
+      ( domain != PF_INET  ) )
     return false;
-  
-  sock = ::socket( PF_INET, SOCK_STREAM, 0 );
+ 
+  sock = ::socket( domain, SOCK_STREAM, 0 );
   if (sock < 0)
   {
     warning( "Could not create socket\n");
     return false;
   }
 
-  struct sockaddr_in name;
+  if (domain == AF_INET) {
+
+    sockaddr_in name;
     
-  name.sin_family = AF_INET;
-  name.sin_port = htons( _port );
-  name.sin_addr.s_addr = htonl(INADDR_ANY);
+    name.sin_family = domain;
+    name.sin_port = htons( _port );
+    name.sin_addr.s_addr = htonl(INADDR_ANY);
     
-  if ( bind( sock, (struct sockaddr*) &name,sizeof( name ) ) < 0 )
-    {
+    if ( bind( sock, (struct sockaddr*) &name,sizeof( name ) ) < 0 ) {
 	  warning("Could not bind to socket\n");
 	  ::close( sock );
 	  sock = -1;
 	  return false;
     }
-    
+  }
+#ifdef INET6
+  else if (domain == AF_INET6) {
+    sockaddr_in6 name;
+
+    name.sin6_family = domain;
+    name.sin6_flowinfo = 0;
+    name.sin6_port = htons(_port);
+    memcpy(&name.sin6_addr, &in6addr_any, sizeof(in6addr_any));
+
+    if ( bind( sock, (struct sockaddr*) &name,sizeof( name ) ) < 0 ) {
+	  warning("Could not bind to socket\n");
+	  ::close( sock );
+	  sock = -1;
+	  return false;
+    }
+  }
+#endif
+
   if ( listen( sock, SOMAXCONN ) < 0 )
     {
 	  warning("Error listening on socket\n");
@@ -535,31 +508,38 @@ bool KServerSocket::init( unsigned short int _port )
   return true;
 }
 
-unsigned short KServerSocket::getPort()
+unsigned short int KServerSocket::port()
 {
   if ( domain != PF_INET )
     return false;
 
-  struct sockaddr_in name; ksize_t len = sizeof(name);
+  ksockaddr_in name; ksize_t len = sizeof(name);
   getsockname(sock, (struct sockaddr *) &name, &len);
-  return ntohs(name.sin_port);
+  return ntohs(get_sin_port(name));
 }
 
-unsigned long KServerSocket::getAddr()
+unsigned long KServerSocket::ipv4_addr()
 {
   if ( domain != PF_INET )
-    return false;
-
-  struct sockaddr_in name; ksize_t len = sizeof(name);
+    return 0;
+  
+  sockaddr_in name; ksize_t len = sizeof(name);
   getsockname(sock, (struct sockaddr *) &name, &len);
-  return ntohl(name.sin_addr.s_addr);
+  if (name.sin_family == AF_INET) // It's IPv4
+    return ntohl(name.sin_addr.s_addr);
+#ifdef INET6
+  else if (name.sin_family == AF_INET6) // It's IPv6 Ah.
+    return 0;
+#endif
+  else // We dunno what it is
+    return 0;
 }
 
 void KServerSocket::slotAccept( int )
 {
   if ( domain == PF_INET )
   {      
-    struct sockaddr_in clientname;
+    ksockaddr_in clientname;
     int new_sock;
     
     ksize_t size = sizeof(clientname);
@@ -593,10 +573,8 @@ KServerSocket::~KServerSocket()
 {
   if ( notifier )
 	delete notifier; 
-  struct sockaddr_un name; ksize_t len = sizeof(name);
-  getsockname(sock, (struct sockaddr *) &name, &len);
-  close( sock );
-  unlink(name.sun_path);                                                       
+  
+  close( sock ); 
 }
 
 #include "ksock.moc"
