@@ -62,7 +62,9 @@ Cambridge, MA 02139, USA.  */
 
 #include <stdlib.h>
 #include <stdio.h>
+#ifndef __USE_GNU
 #define __USE_GNU
+#endif
 
 #if defined(HAVE_ARGZ_H)
 #ifdef __BEGIN_DECLS
@@ -171,7 +173,7 @@ extern int errno;
 #endif
 
 #if UINT_MAX == UINT_MAX_32_BITS
-typedef unsigned nls_uint32;
+typedef unsigned int nls_uint32;
 #else
 # if USHRT_MAX == UINT_MAX_32_BITS
 typedef unsigned short nls_uint32;
@@ -827,7 +829,7 @@ _nl_explode_name (char * name, const char **language,
   if (*language == cp)
     /* This does not make sense: language has to be specified.  Use
        this entry as it is without exploding.  Perhaps it is an alias.  */
-    cp = strchr (*language, '\0');
+    cp = (char*)strchr (*language, '\0');
   else if (cp[0] == '_')
     {
       /* Next is the territory.  */
@@ -1465,7 +1467,7 @@ k_nl_load_domain (struct loaded_l10nfile *domain_file)
       /* The magic number is wrong: not a message catalog file.  */
 #if (defined HAVE_MMAP && defined HAVE_MUNMAP && !defined DISALLOW_MMAP)
       if (use_mmap)
-	munmap ((caddr_t) data, st.st_size);
+	munmap ((char *) data, st.st_size);
       else
 #endif
 	free (data);
@@ -1498,7 +1500,7 @@ k_nl_load_domain (struct loaded_l10nfile *domain_file)
       /* This is an illegal revision.  */
 #if (defined HAVE_MMAP && defined HAVE_MUNMAP && !defined DISALLOW_MMAP)
       if (use_mmap)
-	munmap ((caddr_t) data, st.st_size);
+	munmap ((char *) data, st.st_size);
       else
 #endif
 	free (data);
@@ -1526,7 +1528,7 @@ static size_t maxmap = 0;
 
 /* Prototypes for local functions.  */
 static size_t read_alias_file (const char *fname, int fname_len);
-static void extend_alias_table  (void);
+static void extend_alias_table  ();
 static int alias_compare  (const struct alias_map *map1,
 			   const struct alias_map *map2);
 
@@ -1809,5 +1811,6 @@ k_textdomain (const char *domainname)
   return (char *) k_nl_current_default_domain;
 }
 
-
-
+#ifdef alloca
+#undef alloca
+#endif
