@@ -1,15 +1,32 @@
+/* This file is part of the KDE libraries
+   Copyright (C) 1996 Martynas Kunigelis
+
+   This library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Library General Public
+   License version 2 as published by the Free Software Foundation.
+
+   This library is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Library General Public License for more details.
+
+   You should have received a copy of the GNU Library General Public License
+   along with this library; see the file COPYING.LIB.  If not, write to
+   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+   Boston, MA 02111-1307, USA.
+*/
 /*
- * KProgress -- a progress indicator widget for KDE. 
- * (c) 1996 Martynas Kunigelis
+ * KProgress -- a progress indicator widget for KDE.
  */
- 
-#include "kprogress.h"
-#include "kprogress.h"
 
 #include <qpainter.h>
-#include <qstring.h>
 #include <qpixmap.h>
+#include <qstring.h>
+
+#include "kprogress.h"
+
 #include <kapp.h>
+#include <kstyle.h>
 
 KProgress::KProgress(QWidget *parent, const char *name)
 	: QFrame(parent, name),
@@ -27,7 +44,7 @@ KProgress::KProgress(Orientation orientation, QWidget *parent, const char *name)
 	initialize();
 }
 
-KProgress::KProgress(int minValue, int maxValue, int value, 
+KProgress::KProgress(int minValue, int maxValue, int value,
 					Orientation orientation, QWidget *parent, const char *name)
 	: QFrame(parent, name),
 	QRangeControl(minValue, maxValue, 1, 10, value),
@@ -49,15 +66,23 @@ void KProgress::initialize()
 {
 	//setFrameStyle(QFrame::Panel | QFrame::Sunken);
 	//setLineWidth(2);
-	//setMidLineWidth(2);
+        //setMidLineWidth(2);
+  format_ = "%p%";
 	bar_pixmap = 0;
 	bar_style = Solid;
-	bar_color = kapp->selectColor;
-	bar_text_color = kapp->selectTextColor;
-	text_color = kapp->textColor;
-	setBackgroundColor( kapp->windowColor );
-	setFont(QFont("helvetica", 12, QFont::Bold));
-	text_enabled = TRUE;
+	bar_color = colorGroup().highlight();
+	bar_text_color = colorGroup().highlightedText();
+	text_color = colorGroup().text();
+	setBackgroundMode( PaletteBase );
+	setFont(QFont(QString::fromLatin1("helvetica"), 12, QFont::Bold));
+        text_enabled = TRUE;
+        if(kapp->kstyle()){
+            QBrush b;
+            QPalette p = palette();
+            kapp->kstyle()->getKProgressBackground(colorGroup(), b);
+            p.setBrush(QColorGroup::Base, b);
+            setPalette(p);
+        }
 	adjustStyle();
 }
 
@@ -96,7 +121,7 @@ void KProgress::setOrientation(Orientation orientation)
 		update();
 	}
 }
- 
+
 void KProgress::setValue(int value)
 {
 	QRangeControl::setValue(value);
@@ -135,6 +160,13 @@ QSize KProgress::sizeHint() const
   return s;
 }
 
+QSizePolicy KProgress::sizePolicy() const
+{
+	if ( orientation()==KProgress::Vertical )
+		return QSizePolicy( QSizePolicy::Fixed, QSizePolicy::Expanding );
+	else
+		return QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed );
+}
 
 KProgress::Orientation KProgress::orientation() const
 {
@@ -150,7 +182,7 @@ int KProgress::recalcValue(int range)
 {
 	int abs_value = value() - minValue();
 	int abs_range = maxValue() - minValue();
-	return range * abs_value / abs_range;
+	return abs_range ? range * abs_value / abs_range : 0;
 }
 
 void KProgress::valueChange()
@@ -165,7 +197,7 @@ void KProgress::rangeChange()
 	emit percentageChanged(recalcValue(100));
 }
 
-void KProgress::styleChange(GUIStyle)
+void KProgress::styleChange(QStyle&)
 {
 	adjustStyle();
 }
@@ -187,19 +219,24 @@ void KProgress::adjustStyle()
 
 void KProgress::paletteChange( const QPalette & )
 {
-	bar_color = kapp->selectColor;
-	bar_text_color = kapp->selectTextColor;
-	text_color = kapp->textColor;
-	setBackgroundColor( kapp->windowColor );
+	bar_color = colorGroup().highlight();
+	bar_text_color = colorGroup().highlightedText();
+	text_color = colorGroup().text();
 }
 		
 void KProgress::drawText(QPainter *p)
 {
 	QRect r(contentsRect());
 	//QColor c(bar_color.rgb() ^ backgroundColor().rgb());
-	QString s;
-	
-	s.sprintf("%i%%", recalcValue(100));
+
+  // Rik: Replace the tags '%p', '%v' and '%m' with the current percentage,
+  // the current value and the maximum value respectively.
+  QString s(format_);
+
+  s.replace(QRegExp(QString::fromLatin1("%p")), QString::number(recalcValue(100)));
+  s.replace(QRegExp(QString::fromLatin1("%v")), QString::number(value()));
+  s.replace(QRegExp(QString::fromLatin1("%m")), QString::number(maxValue()));
+
 	p->setPen(text_color);
 	//p->setRasterOp(XorROP);
 	p->drawText(r, AlignCenter, s);
@@ -211,7 +248,8 @@ void KProgress::drawText(QPainter *p)
 
 void KProgress::drawContents(QPainter *p)
 {
-	QRect cr = contentsRect(), er = cr;
+        bool useKStyle = kapp->kstyle() != NULL;
+        QRect cr = contentsRect(), er = cr;
 	fr = cr;
 	QBrush fb(bar_color), eb(backgroundColor());
 
@@ -232,8 +270,15 @@ void KProgress::drawContents(QPainter *p)
 			}
 				
 			p->setBrushOrigin(cr.topLeft());
-			p->fillRect(fr, fb);
-			p->fillRect(er, eb);
+                        if(useKStyle)
+                            kapp->kstyle()->
+                                drawKProgressBlock(p, fr.x(), fr.y(),
+                                                   fr.width(), fr.height(),
+                                                   colorGroup(), &fb);
+                        else
+                            p->fillRect(fr, fb);
+
+                        p->fillRect(er, eb);
 			
 			break;
 			
@@ -257,16 +302,22 @@ void KProgress::drawContents(QPainter *p)
 				max = (cr.height() - margin) / (fr.height() + margin) + 1;
 				num = recalcValue(max);
 			}
-			p->setClipRect(cr.x() + margin, cr.y() + margin, 
+			p->setClipRect(cr.x() + margin, cr.y() + margin,
 						   cr.width() - margin, cr.height() - margin);
 			for (int i = 0; i < num; i++) {
 				p->setBrushOrigin(fr.topLeft());
-				p->fillRect(fr, fb);
-				fr.moveBy(dx, dy);
-			}
+                                if(useKStyle)
+                                    kapp->kstyle()->
+                                        drawKProgressBlock(p, fr.x(), fr.y(),
+                                                           fr.width(), fr.height(),
+                                                           colorGroup(), &fb);
+                                else
+                                    p->fillRect(fr, fb);
+                                fr.moveBy(dx, dy);
+                        }
 			
 			if (num != max) {
-				if (orient == Horizontal) 
+				if (orient == Horizontal)
 					er.setLeft(fr.right() + 1);
 				else
 					er.setBottom(fr.bottom() + 1);
@@ -283,4 +334,17 @@ void KProgress::drawContents(QPainter *p)
 		drawText(p);
 		
 }			
+
+  void
+KProgress::setFormat(const QString & format)
+{
+  format_ = format;
+}
+
+  QString
+KProgress::format() const
+{
+  return format_;
+}
+
 #include "kprogress.moc"
