@@ -25,8 +25,7 @@
 #include <qpainter.h>
 #include <qdrawutil.h>
 #include <qevent.h>
-#include <dither.h>
-#include "kselect.h"
+#include <kimageeffect.h>
 #include "kselect.h"
 
 #define STORE_W 8
@@ -47,7 +46,7 @@ KXYSelector::KXYSelector( QWidget *parent, const char *name )
 	minY = 0;
 	maxX = 100;
 	maxY = 100;
-	store.optimize( TRUE );
+	store.setOptimization( QPixmap::BestOptim );
 	store.resize( STORE_W2, STORE_W2 );
 }
 
@@ -379,11 +378,10 @@ void KGradientSelector::drawContents( QPainter *painter )
 								color1.green() + greenDiff * s / 8,
 								color1.blue() + blueDiff * s / 8 );
 
-	kFSDither dither( ditherPalette, 8 );
-	QImage dImage = dither.dither( image );
+	KImageEffect::dither( image, ditherPalette, 8 );
 
 	QPixmap p;
-	p.convertFromImage( dImage );
+	p.convertFromImage( image );
 
 	painter->drawPixmap( contentsRect().x(), contentsRect().y(), p );
 
