@@ -1,4 +1,4 @@
-
+// This library is distributed under the conditions of the GNU LGPL.
 #ifdef HAVE_CONFIG_H
 #include"config.h"
 #endif
@@ -19,7 +19,7 @@ void kimgio_tiff_read( QImageIO *io )
 	// FIXME: use qdatastream
 
 	// open file
-	tiff = TIFFOpen( io->fileName(), "r" );
+	tiff = TIFFOpen( io->fileName().ascii(), "r" );
 
 	if( tiff == 0 ) {
 		return;
@@ -80,4 +80,5 @@ void kimgio_tiff_write( QImageIO * )
 {
 	// TODO: stub
 }
+
 #endif
